@@ -1,6 +1,7 @@
 import logging
 import os
-from typing import List
+import uuid
+from typing import List, Any
 
 from litellm import get_llm_provider
 
@@ -8,16 +9,33 @@ from astra_assistants import patch, OpenAIWithDefaultKey
 from astra_assistants.astra_assistants_event_handler import AstraEventHandler
 from astra_assistants.tools.tool_interface import ToolInterface
 from astra_assistants.utils import env_var_is_missing, get_env_vars_for_provider
+from astra_assistants.mcp_openai_adapter import MCPOpenAIAAdapter
 
 logger = logging.getLogger(__name__)
 
 class AssistantManager:
-    def __init__(self, instructions: str = None, model: str = "gpt-4o", name: str = "managed_assistant", tools: List[ToolInterface] = None, thread_id: str = None, thread: str = None, assistant_id: str = None, client = None, tool_resources = None):
+    def __init__(self,
+                 instructions: str = None,
+                 model: str = "gpt-4o",
+                 name: str = "managed_assistant",
+                 tools: List[ToolInterface] = None,
+                 thread_id: str = None,
+                 thread: str = None,
+                 assistant_id: str = None,
+                 client = None,
+                 tool_resources = None,
+                 mcp_represenations = None
+                 ):
+
         if instructions is None and assistant_id is None:
             raise Exception("Instructions must be provided if assistant_id is not provided")
         if tools is None:
             tools = []
-        # Only patch if astra token is provided
+
+
+        self.tools = tools
+
+        # Initialize client using the provided client or the default based on environment tokens.
         if client is not None:
             self.client = client
         else:
@@ -31,7 +49,6 @@ class AssistantManager:
                 self.client = OpenAIWithDefaultKey()
         self.model = model
         self.instructions = instructions
-        self.tools = tools
         self.tool_resources = tool_resources
         self.name = name
         self.tool_call_arguments = None
@@ -48,9 +65,25 @@ class AssistantManager:
         elif thread_id is not None:
             self.thread = self.client.beta.threads.retrieve(thread_id)
 
+
+        self.mcp_adapter = None
+        self.register_mcp(mcp_represenations)
+
         logger.info(f'assistant {self.assistant}')
         logger.info(f'thread {self.thread}')
 
+    def register_mcp(self, mcp_representations):
+        # If MCP representations are provided, convert them to tools using the adapter.
+        if mcp_representations is not None:
+            self.mcp_adapter = MCPOpenAIAAdapter(mcp_representations)
+
+            mcp_tools = self.mcp_adapter.get_tools()
+            self.tools.extend(mcp_tools)
+
+            schemas = self.mcp_adapter.get_json_schema_for_tools()
+            assistant = self.client.beta.assistants.update(assistant_id=self.assistant.id, tools=schemas)
+            self.assistant = assistant
+
     def get_client(self):
         return self.client
 
@@ -65,11 +98,10 @@ class AssistantManager:
         for tool in self.tools:
             if hasattr(tool, 'to_function'):
                 tool_holder.append(tool.to_function())
-
         if len(tool_holder) == 0:
             tool_holder = self.tools
 
-        # Create and return the assistant
+        # Create and return the assistant with the combined tool definitions.
         self.assistant = self.client.beta.assistants.create(
             name=self.name,
             instructions=self.instructions,
@@ -77,13 +109,13 @@ class AssistantManager:
             tools=tool_holder,
             tool_resources=self.tool_resources
         )
-        logger.debug("Assistant created:", self.assistant)
+        logger.debug("Assistant created: %s", self.assistant)
         return self.assistant
 
     def create_thread(self):
-        # Create and return a new thread
+        # Create and return a new thread.
         thread = self.client.beta.threads.create()
-        logger.debug("Thread generated:", thread)
+        logger.debug("Thread generated: %s", thread)
         return thread
 
     def stream_thread(self, content, tool_choice = None, thread_id: str = None, thread = None, additional_instructions = None):
@@ -112,7 +144,6 @@ class AssistantManager:
                 "event_handler": event_handler,
                 "additional_instructions": additional_instructions
             }
-            # Conditionally add 'tool_choice' if it's not None
             if tool_choice is not None:
                 args["tool_choice"] = tool_choice
 
@@ -121,8 +152,6 @@ class AssistantManager:
                 for text in stream.text_deltas:
                     yield text
 
-            tool_call_results = None
-            tool_call_arguments = None
             self.tool_call_arguments = event_handler.arguments
             if event_handler.stream is not None:
                 if event_handler.tool_call_results is not None:
@@ -133,7 +162,7 @@ class AssistantManager:
         except Exception as e:
             logger.error(e)
             raise e
-        
+
     async def run_thread(self, content, tool = None, thread_id: str = None, thread = None, additional_instructions = None):
         if thread_id is not None:
             thread = self.client.beta.threads.retrieve(thread_id)
@@ -142,10 +171,15 @@ class AssistantManager:
 
         assistant = self.assistant
         event_handler = AstraEventHandler(self.client)
+
         tool_choice = None
         if tool is not None:
             event_handler.register_tool(tool)
             tool_choice = tool.tool_choice_object()
+
+        for tool in self.tools:
+            event_handler.register_tool(tool)
+
         try:
             self.client.beta.threads.messages.create(
                 thread_id=thread.id, role="user", content=content
@@ -156,15 +190,14 @@ class AssistantManager:
                 "event_handler": event_handler,
                 "additional_instructions": additional_instructions
             }
-            # Conditionally add 'tool_choice' if it's not None
             if tool_choice is not None:
                 args["tool_choice"] = tool_choice
 
             text = ""
-            with self.client.beta.threads.runs.create_and_stream(**args) as stream:
+            with self.client.beta.threads.runs.stream(**args) as stream:
                 for part in stream.text_deltas:
                     text += part
-                    
+
             tool_call_results = None
             if event_handler.stream is not None:
                 with event_handler.stream as stream:
@@ -172,17 +205,22 @@ class AssistantManager:
                         text += part
 
                     tool_call_results = event_handler.tool_call_results
-                    file_search = event_handler.file_search
+                    if tool_call_results is not None:
+                        file_search = event_handler.file_search
 
-                    tool_call_results['file_search'] = file_search
-                    tool_call_results['text'] = text
-                    tool_call_results['arguments'] = event_handler.arguments
+                        tool_call_results['file_search'] = file_search
+                        tool_call_results['text'] = text
+                        tool_call_results['arguments'] = event_handler.arguments
+                    else:
+                        print("event_handler.stream is not None but tool_call_results is None, bug?")
 
                     logger.info(tool_call_results)
-                    tool_call_results
             if tool_call_results is not None:
                 return tool_call_results
             return {"text": text, "file_search": event_handler.file_search}
         except Exception as e:
             logger.error(e)
-            raise e
\ No newline at end of file
+            raise e
+
+    def shutdown(self):
+        self.mcp_adapter.shutdown()
\ No newline at end of file
